' Fig. 15.13: RandomLetters.vb
' Writes a random letter to a label
Imports System.Threading

Public Class RandomLetters
   Private Shared generator As New Random() ' for random letters
   Private suspended As Boolean = False ' true if thread is suspended
   Private output As Label ' Label for output
   Private threadName As String ' name of the current thread

   ' constructor
   Public Sub New(ByVal label As Label)
      output = label
   End Sub ' New

   ' delegate that allows method DisplayCharacter to be called
   ' in the thread that creates and maintains the GUI       
   Delegate Sub DisplayDelegate(ByVal displayChar As Char)

   ' method DisplayCharacter sets the Label's Text property
   ' in a thread-safe manner
   Private Sub DisplayCharacter(ByVal displayChar As Char)
      ' output character in Label
      output.Text = threadName + ": " + displayChar
   End Sub ' DisplayCharacter

   ' place random characters in GUI
   Public Sub Run()
      ' get name of executing thread
      threadName = Thread.CurrentThread.Name

      While True ' infinite loop; will be terminated from outside
         ' sleep for up to 1 second
         Thread.Sleep(generator.Next(1001))

         SyncLock Me ' obtain lock
            While suspended ' loop until not suspended
               Monitor.Wait(Me) ' suspend thread execution
            End While
         End SyncLock

         ' select random uppercase letter
         Dim displayChar As Char = ChrW(generator.Next(26) + 65)

         ' display character on corresponding Label
         output.Invoke(New DisplayDelegate(AddressOf DisplayCharacter), _
            New Object() {displayChar})
      End While
   End Sub ' Run

   ' change the suspended/running state
   Public Sub Toggle()
      suspended = Not suspended ' toggle bool controlling state

      ' change label color on suspend/resume
      If suspended Then
         output.BackColor = Color.Red
      Else
         output.BackColor = Color.LightGreen
      End If

      SyncLock Me ' obtain lock
         If Not suspended Then ' if thread resumed
            Monitor.Pulse(Me)
         End If
      End SyncLock
   End Sub ' Toggle
End Class ' RandomLetters

'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************