' Fig. 17.28: FrmAgent.vb
' Microsoft Agent demonstration.
Imports System.IO

Public Class FrmAgent
   ' current agent object
   Private speaker As AgentObjects.IAgentCtlCharacter

   ' parameterless constructor
   Public Sub New()
      InitializeComponent()

      ' initialize the characters
      Try
         ' load characters into agent object
         mainAgent.Characters.Load("Genie", _
            "C:\windows\msagent\chars\Genie.acs")
         mainAgent.Characters.Load("Merlin", _
            "C:\windows\msagent\chars\Merlin.acs")
         mainAgent.Characters.Load("Peedy", _
            "C:\windows\msagent\chars\Peedy.acs")
         mainAgent.Characters.Load("Robby", _
            "C:\windows\msagent\chars\Robby.acs")

         ' set current character to Genie and show him
         speaker = mainAgent.Characters("Genie")
         GetAnimationNames() ' obtain an animation name list
         speaker.Show(0) ' display Genie
         cboCharacter.SelectedText = "Genie"
      Catch fileNotFound As FileNotFoundException
         MessageBox.Show("Invalid character location", _
            "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
      End Try
   End Sub ' New

   ' event handler for Speak Button
   Private Sub btnSpeak_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles btnSpeak.Click
      ' if TextBox is empty, have the character ask
      ' user to type the words into the TextBox otherwise,
      ' have the character say the words in the TextBox
      If txtSpeech.Text = "" Then
         speaker.Speak("Please, type the words you want me to speak", "")
      Else
         speaker.Speak(txtSpeech.Text, "")
      End If
   End Sub ' btnSpeak_Click

   ' event handler for Agent control's ClickEvent
   Private Sub mainAgent_ClickEvent(ByVal sender As Object, _
      ByVal e As AxAgentObjects._AgentEvents_ClickEvent) _
      Handles mainAgent.ClickEvent

      speaker.Play("Confused")
      speaker.Speak("Why are you poking me?", "")
      speaker.Play("RestPose")
   End Sub ' mainAgent_ClickEvent

   ' ComboBox changed event, switch active agent character
   Private Sub cboCharacter_SelectedIndexChanged( _
      ByVal sender As System.Object, ByVal e As System.EventArgs) _
      Handles cboCharacter.SelectedIndexChanged

      ChangeCharacter(cboCharacter.Text)
   End Sub ' cboCharacter_SelectedIndexChanged

   ' utility method to change characters
   Private Sub ChangeCharacter(ByVal name As String)
      speaker.StopAll("Play")
      speaker.Hide(0)
      speaker = mainAgent.Characters(Name)

      ' regenerate animation name list
      GetAnimationNames()
      speaker.Show(0)
   End Sub ' ChangeCharacter

   ' get animation names and store in ArrayList
   Private Sub GetAnimationNames()
      ' ensure thread safety
      SyncLock (Me)
         ' get animation names
         Dim enumerator As IEnumerator = mainAgent.Characters( _
            speaker.Name).AnimationNames.GetEnumerator()

         Dim voiceString As String

         ' clear cboActions
         cboActions.Items.Clear()
         speaker.Commands.RemoveAll()

         ' copy enumeration to ArrayList
         While enumerator.MoveNext()
            ' remove underscores in speech string
            voiceString = enumerator.Current.ToString()
            voiceString = voiceString.Replace("_", "underscore")

            cboActions.Items.Add(enumerator.Current)

            ' add all animations as voice enabled commands
            speaker.Commands.Add(enumerator.Current.ToString(), _
               enumerator.Current, voiceString, True, False)
         End While

         ' add custom command
         speaker.Commands.Add("MoveToMouse", "MoveToMouse", _
            "MoveToMouse", True, True)
      End SyncLock
   End Sub ' GetAnimationNames

   ' user selects new action
   Private Sub cboActions_SelectedIndexChanged( _
      ByVal sender As System.Object, ByVal e As System.EventArgs) _
      Handles cboActions.SelectedIndexChanged

      speaker.StopAll("Play")
      speaker.Play(cboActions.Text)
      speaker.Play("RestPose")
   End Sub ' cboActions_SelectedIndexChanged

   ' event handler for Agent commands
   Private Sub mainAgent_Command(ByVal sender As Object, _
      ByVal e As AxAgentObjects._AgentEvents_CommandEvent) _
      Handles mainAgent.Command
      ' get UserInput object
      Dim command As AgentObjects.IAgentCtlUserInput = _
         CType(e.userInput, AgentObjects.IAgentCtlUserInput)

      ' change character if user speaks character name
      If command.Voice = "Peedy" OrElse command.Voice = "Robby" OrElse _
         command.Voice = "Merlin" OrElse command.Voice = "Genie" Then
         ChangeCharacter(command.Voice)
         Return
      End If

      ' send agent to mouse
      If command.Voice = "MoveToMouse" Then
         speaker.MoveTo( _
            Convert.ToInt16(Windows.Forms.Cursor.Position.X - 60), _
            Convert.ToInt16(Windows.Forms.Cursor.Position.Y - 60), 5)
         Return
      End If

      ' play new animation
      speaker.StopAll("Play")
      speaker.Play(command.Name)
   End Sub ' mainAgent_Command
End Class ' FrmAgent

'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************
