' Fig. 18.12: FrmCreditInquiry.vb
' Read a file sequentially and display contents based on
' account type specified by user (credit, debit or zero balances).
Imports System.IO
Imports BankLibrary

Public Class FrmCreditInquiry
   Private input As FileStream ' maintains the connection to the file
   Private fileReader As StreamReader ' reads data from text file

   ' name of file that stores credit, debit and zero balances
   Private fileName As String

   ' invoked when user clicks Open File button
   Private Sub btnOpen_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles btnOpen.Click
      ' create dialog box enabling user to open file
      Dim fileChooser As New OpenFileDialog()
      Dim result As DialogResult = fileChooser.ShowDialog()

      ' exit event handler if user clicked Cancel
      If result = Windows.Forms.DialogResult.Cancel Then
         Return
      End If

      fileName = fileChooser.FileName ' get name from user

      ' show error if user specified invalid file
      If fileName = "" Or fileName Is Nothing Then
         MessageBox.Show("Invalid File Name", "Error", _
            MessageBoxButtons.OK, MessageBoxIcon.Error)
      Else
         ' create FileStream to obtain read access to file
         input = New FileStream(fileName, FileMode.Open, FileAccess.Read)

         ' set file from where data is read
         fileReader = New StreamReader(input)

         ' enable all GUI buttons, except for Open File button
         btnOpen.Enabled = False
         btnCredit.Enabled = True
         btnDebit.Enabled = True
         btnZero.Enabled = True
      End If
   End Sub ' btnOpen_Click

   ' invoked when user clicks credit balances,
   ' debit balances or zero balances button
   Private Sub getBalances_Click( _
      ByVal sender As System.Object, ByVal e As System.EventArgs) _
      Handles btnCredit.Click, btnZero.Click, btnDebit.Click
      ' convert sender explicitly to object of type button
      Dim senderButton As Button = CType(sender, Button)

      ' get text from clicked Button, which stores account type
      Dim accountType As String = senderButton.Text

      ' read and display file information
      Try
         ' go back to the beginning of the file
         input.Seek(0, SeekOrigin.Begin)

         txtDisplay.Text = "The accounts are:" & vbCrLf

         ' traverse file until end of file
         While True
            ' will store individual pieces of data
            Dim inputFields() As String

            Dim record As Record ' store each Record as file is read
            Dim balance As Decimal ' store each Record's balance

            ' get next Record available in file
            Dim inputRecord As String = fileReader.ReadLine()

            ' when at the end of file, exit method
            If inputRecord Is Nothing Then
               Return
            End If

            inputFields = inputRecord.Split(","c) ' parse input

            ' create Record from input
            record = New Record(Convert.ToInt32( _
               inputFields(0)), inputFields(1), inputFields(2), _
               Convert.ToDecimal(inputFields(3)))

            ' store record's last field in balance
            balance = record.Balance

            ' determine whether to display balance
            If ShouldDisplay(balance, accountType) Then
               ' display record
               Dim output As String = _
                  record.Account & vbTab & _
                  record.FirstName & vbTab & _
                  record.LastName & vbTab

               ' display balance with correct monetary format
               output &= String.Format("{0:F}", balance) & vbCrLf

               txtDisplay.Text &= output ' copy output to screen
            End If
         End While
         ' handle exception when file cannot be read
      Catch ex As IOException
         MessageBox.Show("Cannot Read File", "Error", _
            MessageBoxButtons.OK, MessageBoxIcon.Error)
      End Try
   End Sub ' getBalances_Click

   ' determine whether to display given record
   Private Function ShouldDisplay(ByVal balance As Decimal, _
      ByVal accountType As String) As Boolean

      If balance > 0 Then
         ' display credit balances
         If accountType = "Credit Balances" Then
            Return True
         End If
      ElseIf balance < 0 Then
         ' display debit balances
         If accountType = "Debit Balances" Then
            Return True
         End If
      Else ' balance == 0
         ' display zero balances
         If accountType = "Zero Balances" Then
            Return True
         End If
      End If

      Return False
   End Function ' ShouldDisplay

   ' invoked when user clicks Done button
   Private Sub btnDone_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles btnDone.Click
      ' determine whether file exists
      If input IsNot Nothing Then
         ' close file and StreamReader
         Try
            input.Close()
            fileReader.Close()
            ' handle exception if FileStream does not exist
         Catch ex As IOException
            ' notify user of error closing file
            MessageBox.Show("Cannot close file", "Error", _
               MessageBoxButtons.OK, MessageBoxIcon.Error)
         End Try
      End If

      Application.Exit()
   End Sub ' btnDone_Click
End Class ' FrmCreditInquiry

'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************