' Fig. 23.2: FrmChatClient.vb
' Set up a client that will send information to and
' read information from a server.
Imports System.Threading
Imports System.Net.Sockets
Imports System.IO

Public Class FrmChatClient
   Private output As NetworkStream ' stream for receiving data           
   Private writer As BinaryWriter ' facilitates writing to the stream    
   Private reader As BinaryReader ' facilitates reading from the stream  
   Private readThread As Thread ' Thread for processing incoming messages
   Private message As String = ""

   ' initialize thread for reading
   Private Sub FrmChatClient_Load(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles MyBase.Load

      readThread = New Thread(New ThreadStart(AddressOf RunClient))
      readThread.Start()
   End Sub ' FrmChatClient_Load

   ' close all threads associated with this application
   Private Sub FrmChatClient_FormClosing(ByVal sender As System.Object, _
      ByVal e As System.Windows.Forms.FormClosingEventArgs) _
      Handles MyBase.FormClosing

      System.Environment.Exit(System.Environment.ExitCode)
   End Sub ' FrmChatClient_FormClosing

   ' Delegate that allows method DisplayMessage to be called
   ' in the thread that creates and maintains the GUI       
   Private Delegate Sub DisplayDelegate(ByVal message As String)

   ' method DisplayMessage sets txtDisplay's Text property
   ' in a thread-safe manner
   Private Sub DisplayMessage(ByVal message As String)
      ' if modifying txtDisplay is not thread safe
      If txtDisplay.InvokeRequired Then
         ' use inherited method Invoke to execute DisplayMessage
         ' via a Delegate                                       
         Invoke(New DisplayDelegate(AddressOf DisplayMessage), _
            New Object() {message})
         ' OK to modify txtDisplay in current thread
      Else
         txtDisplay.Text &= message
      End If
   End Sub ' DisplayMessage

   ' Delegate that allows method DisableInput to be called 
   ' in the thread that creates and maintains the GUI
   Private Delegate Sub DisableInputDelegate(ByVal value As Boolean)

   ' method DisableInput sets txtInput's ReadOnly property
   ' in a thread-safe manner
   Private Sub DisableInput(ByVal value As Boolean)
      ' if modifying txtInput is not thread safe
      If txtInput.InvokeRequired Then
         ' use inherited method Invoke to execute DisableInput
         ' via a Delegate                                     
         Invoke(New DisableInputDelegate(AddressOf DisableInput), _
            New Object() {value})
         ' OK to modify txtInput in current thread
      Else
         txtInput.ReadOnly = value
      End If
   End Sub ' DisableInput

   ' sends text the user typed to server
   Private Sub txtInput_KeyDown(ByVal sender As System.Object, _
      ByVal e As System.Windows.Forms.KeyEventArgs) _
      Handles txtInput.KeyDown

      Try
         If e.KeyCode = Keys.Enter And txtInput.ReadOnly = False Then
            writer.Write("CLIENT>>> " & txtInput.Text)
            txtDisplay.Text &= vbCrLf & "CLIENT>>> " & txtInput.Text
            txtInput.Clear()
         End If
      Catch ex As SocketException
         txtDisplay.Text &= vbCrLf & "Error writing object"
      End Try
   End Sub ' txtInput_KeyDown

   ' connect to server and display server-generated text
   Public Sub RunClient()
      Dim client As TcpClient

      ' instantiate TcpClient for sending data to server
      Try
         DisplayMessage("Attempting connection" & vbCrLf)

         ' Step 1: create TcpClient and connect to server
         client = New TcpClient()
         client.Connect("localhost", 50000)

         ' Step 2: get NetworkStream associated with TcpClient
         output = client.GetStream()

         ' create objects for writing and reading across stream
         writer = New BinaryWriter(output)
         reader = New BinaryReader(output)

         DisplayMessage(vbCrLf & "Got I/O streams" & vbCrLf)
         DisableInput(False) ' enable txtInput

         ' loop until server signals termination
         Do
            ' Step 3: processing phase
            Try
               ' read message from server        
               message = reader.ReadString()
               DisplayMessage(vbCrLf & message)
            Catch ex As Exception
               ' handle exception if error in reading server data
               System.Environment.Exit(System.Environment.ExitCode)
            End Try
         Loop While message <> "SERVER>>> TERMINATE"

         ' Step 4: close connection
         writer.Close()
         reader.Close()
         output.Close()
         client.Close()

         Application.Exit()
      Catch ex As Exception
         ' handle exception if error in establishing connection
         MessageBox.Show(ex.ToString(), "Connection Error", _
            MessageBoxButtons.OK, MessageBoxIcon.Error)
         System.Environment.Exit(System.Environment.ExitCode)
      End Try
   End Sub ' RunClient
End Class ' FrmChatClient


'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************