' Fig. 23.6: FrmTicTacToeClient.vb
' Client for the TicTacToe program.
Imports System.Drawing
Imports System.Net.Sockets
Imports System.Threading
Imports System.IO

Public Class FrmTicTacToeClient
   Private board(,) As Square ' local representation of the game board
   Private currentSquareValue As Square ' Square that this player chose   
   Private outputThread As Thread ' Thread for receiving data from server
   Private connection As TcpClient ' client to establish connection      
   Private stream As NetworkStream ' network data stream                 
   Private writer As BinaryWriter ' facilitates writing to the stream    
   Private reader As BinaryReader ' facilitates reading from the stream  
   Private myMark As Char ' player's mark on the board                   
   Private myTurn As Boolean ' is it this player's turn?                    
   Private brush As SolidBrush ' brush for drawing X's and O's           
   Private done As Boolean = False ' True when game is over

   ' initialize variables and thread for connecting to server
   Private Sub FrmTicTacToeClient_Load(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles MyBase.Load

      board = New Square(3, 3) {}

      ' create 9 Square objects and place them on the board
      board(0, 0) = New Square(pnlBoard0, " "c, 0)
      board(0, 1) = New Square(pnlBoard1, " "c, 1)
      board(0, 2) = New Square(pnlBoard2, " "c, 2)
      board(1, 0) = New Square(pnlBoard3, " "c, 3)
      board(1, 1) = New Square(pnlBoard4, " "c, 4)
      board(1, 2) = New Square(pnlBoard5, " "c, 5)
      board(2, 0) = New Square(pnlBoard6, " "c, 6)
      board(2, 1) = New Square(pnlBoard7, " "c, 7)
      board(2, 2) = New Square(pnlBoard8, " "c, 8)

      ' create a SolidBrush for writing on the Squares
      brush = New SolidBrush(Color.Black)

      ' make connection to server and get the associated
      ' network stream                                  
      connection = New TcpClient("localhost", 50000)
      stream = connection.GetStream()
      writer = New BinaryWriter(stream)
      reader = New BinaryReader(stream)

      ' start a new thread for sending and receiving messages
      outputThread = New Thread(New ThreadStart(AddressOf Run))
      outputThread.Start()
   End Sub ' FrmTicTacToeClient_Load

   ' repaint the Squares
   Private Sub FrmTicTacToeClient_Paint(ByVal sender As System.Object, _
      ByVal e As System.Windows.Forms.PaintEventArgs) _
      Handles MyBase.Paint

      PaintSquares()
   End Sub ' FrmTicTacToeClient_Paint

   ' game is over
   Private Sub FrmTicTacToeClient_FormClosing( _
      ByVal sender As System.Object, _
      ByVal e As System.Windows.Forms.FormClosingEventArgs) _
      Handles MyBase.FormClosing

      done = True
      System.Environment.Exit(System.Environment.ExitCode)
   End Sub ' FrmTicTacToeClient_FormClosing

   ' Delegate that allows method DisplayMessage to be called
   ' in the thread that creates and maintains the GUI       
   Private Delegate Sub DisplayDelegate(ByVal message As String)

   ' method DisplayMessage sets txtDisplay's Text property
   ' in a thread-safe manner
   Private Sub DisplayMessage(ByVal message As String)
      ' if modifying txtDisplay is not thread safe
      If txtDisplay.InvokeRequired Then
         ' use inherited method Invoke to execute DisplayMessage
         ' via a Delegate                                       
         Invoke(New DisplayDelegate(AddressOf DisplayMessage), _
            New Object() {message})
         ' OK to modify txtDisplay in current thread
      Else
         txtDisplay.Text &= message
      End If
   End Sub ' DisplayMessage

   ' Delegate that allows method ChangeIdLabel to be called
   ' in the thread that creates and maintains the GUI       
   Private Delegate Sub ChangeIdLabelDelegate(ByVal message As String)

   ' method ChangeIdLabel sets txtDisplay's Text property
   ' in a thread-safe manner
   Private Sub ChangeIdLabel(ByVal label As String)
      ' if modifying lblId is not thread safe
      If lblId.InvokeRequired Then
         ' use inherited method Invoke to execute ChangeIdLabel
         ' via a Delegate                                       
         Invoke(New ChangeIdLabelDelegate(AddressOf ChangeIdLabel), _
            New Object() {label})
         ' OK to modify lblId in current thread
      Else
         lblId.Text = label
      End If
   End Sub ' ChangeIdLabel

   ' draws the mark of each square
   Public Sub PaintSquares()
      Dim g As Graphics

      ' draw the appropriate mark on each panel
      For row As Integer = 0 To 2

         For column As Integer = 0 To 2
            ' get the Graphics for each Panel                    
            g = board(row, column).SquarePanel.CreateGraphics()

            ' draw the appropriate letter on the panel        
            g.DrawString(board(row, column).Mark.ToString(), _
               pnlBoard0.Font, brush, 10, 8)
         Next column

      Next row
   End Sub ' PaintSquares

   ' send location of the clicked square to server
   Private Sub square_MouseUp(ByVal sender As System.Object, _
      ByVal e As System.Windows.Forms.MouseEventArgs) _
      Handles pnlBoard0.MouseUp, pnlBoard5.MouseUp, pnlBoard4.MouseUp, _
      pnlBoard3.MouseUp, pnlBoard2.MouseUp, pnlBoard1.MouseUp, _
      pnlBoard8.MouseUp, pnlBoard7.MouseUp, pnlBoard6.MouseUp

      ' for each square check if that square was clicked
      For row As Integer = 0 To 2

         For column As Integer = 0 To 2
            If board(row, column).SquarePanel.Equals(sender) Then
               currentSquareValue = board(row, column)

               ' send the move to the server                     
               SendClickedSquare(board(row, column).Location)
            End If
         Next column
      Next row
   End Sub ' square_MouseUp

   ' control thread that allows continuous update of the
   ' TextBox display
   Public Sub Run()
      myMark = reader.ReadChar() ' first get players's mark (X or O)
      ChangeIdLabel("You are player """ & myMark & """")

      If myMark = "X"c Then
         myTurn = True
      Else
         myTurn = False
      End If

      ' process incoming messages
      Try
         ' receive messages sent to client       
         While Not done
            ProcessMessage(reader.ReadString())
         End While
      Catch ex As IOException
         MessageBox.Show("Server is down, game over", "Error", _
            MessageBoxButtons.OK, MessageBoxIcon.Error)
      End Try
   End Sub ' Run

   ' process messages sent to client
   Public Sub ProcessMessage(ByVal message As String)
      ' if the move the player sent to the server is valid
      ' update the display, set that square's mark to be
      ' the mark of the current player and repaint the board
      If message = "Valid move." Then
         DisplayMessage("Valid move, please wait." & vbCrLf)
         currentSquareValue.Mark = myMark
         PaintSquares()
      ElseIf message = "Invalid move, try again." Then
         ' if the move is invalid, display that and it is now
         ' this player's turn again
         DisplayMessage(message & vbCrLf)
         myTurn = True
      ElseIf message = "Opponent moved." Then
         ' if opponent moved, find location of their move
         Dim location As Integer = reader.ReadInt32()

         ' set that square to have the opponents mark and
         ' repaint the board
         If myMark = "X"c Then
            board(location \ 3, location Mod 3).Mark = "O"c
         Else
            board(location \ 3, location Mod 3).Mark = "X"c
         End If
         PaintSquares()

         DisplayMessage("Opponent moved.  Your turn." & vbCrLf)

         ' it is now this player's turn
         myTurn = True
      Else
         DisplayMessage(message & vbCrLf) ' display message
      End If
   End Sub ' ProcessMessage

   ' sends the server the number of the clicked square
   Public Sub SendClickedSquare(ByVal location As Integer)
      ' if it is the current player's move right now
      If myTurn Then
         ' send the location of the move to the server
         writer.Write(location)

         ' it is now the other player's turn
         myTurn = False
      End If
   End Sub ' SendClickedSquare

   ' write-only property for the current square
   Public WriteOnly Property CurrentSquare() As Square
      Set(ByVal value As Square)
         currentSquareValue = value
      End Set
   End Property ' CurrentSquare
End Class ' FrmTicTacToeClient

'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************