' Fig. 24.4: LinkedListLibrary.vb
' Class ListNode and class List declarations.

Public Class ListNode
   Private dataValue As Object ' stores data for this node
   Private nextNodeReference As ListNode ' stores reference to next node

   ' constructor to create a ListNode that contains data
   ' and is the last node in the List
   Public Sub New(ByVal data As Object)
      MyClass.New(data, Nothing) ' invokes the other constructor
   End Sub ' New

   ' constructor to create a ListNode that contains data
   ' and refers to the next ListNode in the List
   Public Sub New(ByVal data As Object, ByVal nextNode As ListNode)
      dataValue = data
      nextNodeReference = nextNode
   End Sub ' New

   ' property Next
   Public Property NextNode() As ListNode
      Get
         Return nextNodeReference
      End Get
      Set(ByVal value As ListNode)
         nextNodeReference = value
      End Set
   End Property ' NextNode

   ' property Data
   Public ReadOnly Property Data() As Object
      Get
         Return dataValue
      End Get
   End Property ' Data
End Class ' ListNode

' class List declaration
Public Class List
   Private firstNode As ListNode
   Private lastNode As ListNode
   Private name As String ' string like "list" to display

   ' construct empty List with "list" as its name
   Public Sub New()
      MyClass.New("list") ' invokes the other constructor
   End Sub ' New

   ' construct empty List with specified name
   Public Sub New(ByVal listName As String)
      name = listName
      firstNode = Nothing
      lastNode = Nothing
   End Sub ' New

   ' Insert object at front of List. If List is empty, firstNode and 
   ' lastNode will refer to same object. Otherwise, firstNode refers
   ' to new node and the new node refers to the previous first node.
   Public Sub InsertAtFront(ByVal insertItem As Object)
      If IsEmpty() Then
         lastNode = New ListNode(insertItem)
         firstNode = lastNode
      Else
         firstNode = New ListNode(insertItem, firstNode)
      End If
   End Sub ' InsertAtFront

   ' Insert object at end of List. If List is empty, 
   ' firstNode and lastNode will refer to same object.
   ' Otherwise, lastNode's NextNode property refers to new node.
   Public Sub InsertAtBack(ByVal insertItem As Object)
      If IsEmpty() Then
         lastNode = New ListNode(insertItem)
         firstNode = lastNode
      Else
         lastNode.NextNode = New ListNode(insertItem)
         lastNode = lastNode.NextNode
      End If
   End Sub ' InsertAtBack

   ' remove first node from List
   Public Function RemoveFromFront() As Object
      If IsEmpty() Then
         Throw New EmptyListException(name)
      End If

      Dim removeItem As Object = firstNode.Data ' retrieve data

      ' reset firstNode and lastNode references
      If firstNode.Equals(lastNode) Then
         firstNode = Nothing
         lastNode = Nothing
      Else
         firstNode = firstNode.NextNode
      End If

      Return removeItem ' return removed data
   End Function ' RemoveFromFront

   ' remove last node from List
   Public Function RemoveFromBack() As Object
      If IsEmpty() Then
         Throw New EmptyListException(name)
      End If

      Dim removeItem As Object = lastNode.Data ' retrieve data

      ' reset firstNode and lastNode references
      If firstNode.Equals(lastNode) Then
         firstNode = Nothing
         lastNode = Nothing
      Else
         Dim current As ListNode = firstNode

         ' loop while current node is not lastNode
         While Not current.NextNode.Equals(lastNode)
            current = current.NextNode ' move to next node
         End While

         ' current is new lastNode
         lastNode = current
         current.NextNode = Nothing
      End If

      Return removeItem ' return removed data
   End Function ' RemoveFromBack

   ' return True if List is empty
   Public Function IsEmpty() As Boolean
      Return firstNode Is Nothing
   End Function ' IsEmpty

   ' output List contents
   Public Sub Print()
      If IsEmpty() Then
         Console.WriteLine("Empty " & name)
         Return
      End If

      Console.Write("The " & name & " is: ")

      Dim current As ListNode = firstNode

      ' output current node data while not at end of list
      While current IsNot Nothing
         Console.Write(current.Data & " ")
         current = current.NextNode
      End While

      Console.WriteLine(vbCrLf)
   End Sub ' Print 
End Class ' List

' class EmptyListException declaration
Public Class EmptyListException : Inherits ApplicationException
   Public Sub New(ByVal name As String)
      MyBase.New("The " & name & " is empty")
   End Sub ' New
End Class ' EmptyListException


'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************