' Fig. 24.20: BinaryTreeLibrary.vb
' Declaration of class TreeNode and class Tree.

Public Class TreeNode
   Private leftNodeReference As TreeNode ' link to left child
   Private dataValue As Integer ' dataValue stored in node
   Private rightNodeReference As TreeNode ' link to right child

   ' initialize dataValue and make this a leaf node
   Public Sub New(ByVal nodeData As Integer)
      dataValue = nodeData
      leftNodeReference = Nothing
      rightNodeReference = Nothing
   End Sub ' New

   ' LeftNode property
   Public Property LeftNode() As TreeNode
      Get
         Return leftNodeReference
      End Get
      Set(ByVal value As TreeNode)
         leftNodeReference = value
      End Set
   End Property ' LeftNode

   ' Data property
   Public Property Data() As Integer
      Get
         Return dataValue
      End Get
      Set(ByVal value As Integer)
         dataValue = value
      End Set
   End Property ' Data

   ' RightNode property
   Public Property RightNode() As TreeNode
      Get
         Return rightNodeReference
      End Get
      Set(ByVal value As TreeNode)
         rightNodeReference = value
      End Set
   End Property ' RightNode

   ' recursively insert TreeNode into Tree that contains nodes;
   ' ignore duplicate values
   Public Sub Insert(ByVal insertValue As Integer)
      If insertValue < data Then ' insert in left subtree
         ' insert new TreeNode
         If LeftNode Is Nothing Then
            LeftNode = New TreeNode(insertValue)
         Else ' continue traversing left subtree
            LeftNode.Insert(insertValue)
         End If
      ElseIf insertValue > data Then ' insert in right subtree
         ' insert new TreeNode
         If RightNode Is Nothing Then
            RightNode = New TreeNode(insertValue)
         Else ' continue traversing right subtree
            RightNode.Insert(insertValue)
         End If
      End If
   End Sub ' Insert
End Class ' TreeNode

' class Tree declaration
Public Class Tree
   Private root As TreeNode

   ' construct an empty Tree of integers
   Public Sub New()
      root = Nothing
   End Sub ' New

   ' Insert a new node in the binary search tree.
   ' If the root node is null, create the root node here.
   ' Otherwise, call the insert method of class TreeNode.
   Public Sub InsertNode(ByVal insertValue As Integer)
      If root Is Nothing Then
         root = New TreeNode(insertValue)
      Else
         root.Insert(insertValue)
      End If
   End Sub ' InsertNode

   ' begin preorder traversal
   Public Sub PreorderTraversal()
      PreorderHelper(root)
   End Sub ' PreorderTraversal 

   ' recursive method to perform preorder traversal
   Private Sub PreorderHelper(ByVal node As TreeNode)
      If node Is Nothing Then
         Return
      End If

      ' output node data
      Console.Write(node.Data & " ")

      ' traverse left subtree
      PreorderHelper(node.LeftNode)

      ' traverse right subtree
      PreorderHelper(node.RightNode)
   End Sub ' PreorderHelper

   ' begin inorder traversal
   Public Sub InorderTraversal()
      InorderHelper(root)
   End Sub ' InorderTraversal

   ' recursive method to perform inorder traversal
   Private Sub InorderHelper(ByVal node As TreeNode)
      If node Is Nothing Then
         Return
      End If

      ' traverse left subtree
      InorderHelper(node.LeftNode)

      ' output node data
      Console.Write(node.Data & " ")

      ' traverse right subtree
      InorderHelper(node.RightNode)
   End Sub ' InorderHelper

   ' begin postorder traversal
   Public Sub PostorderTraversal()
      PostorderHelper(root)
   End Sub ' PostorderTraversal

   ' recursive method to perform postorder traversal
   Private Sub PostorderHelper(ByVal node As TreeNode)
      If node Is Nothing Then
         Return
      End If

      ' traverse left subtree
      PostorderHelper(node.LeftNode)

      ' traverse right subtree
      PostorderHelper(node.RightNode)

      ' output node data
      Console.Write(node.Data & " ")
   End Sub ' PostorderHelper
End Class ' Tree

'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************